{% extends "base.html" %}

{% block extrahead %}
<meta name="theme-color" content="#212121">

<link rel='icon' href='/assets/images/favicon.ico'>

<meta name='apple-mobile-web-app-capable' content='yes'>
<meta name='mobile-web-app-capable' content='yes'>
<link rel="manifest" href="/manifest.json">
{% endblock %}